@@ -1,13 +1,9 @@
 #' Pedigree utilities
 #'
-#' Various utility functions for `ped` objects
-#'
-#' The functions `pedsize()`, `hasUnbrokenLoops()`, `hasInbredFounders()` and
-#' `hasSelfing()` allow as input either a single `ped` object or a list of such.
-#' In the latter case each function returns TRUE if it is TRUE for any of the
-#' components.
+#' Various utility functions for `ped` objects.
 #'
 #' @param x A `ped` object, or (in some functions - see Details) a list of such.
+#' @param maxComp A logical, by default TRUE. See Value.
 #' @param chromType Either "autosomal" (default) or "x".
 #'
 #' @return
@@ -15,6 +11,13 @@
 #' * `pedsize(x)` returns the number of pedigree members in each component of
 #' `x`.
 #'
+#' * `generations(x)` returns the number of generations in `x`, defined as the
+#' number of individuals in the longest line of parent-child links. (Note that
+#' this definition is valid also if `x` has loops.) If `x` has multiple
+#' components, the output depends on the parameter `maxComp`. If this is FALSE,
+#' the output is a vector containing the result for each component. If TRUE
+#' (default), only the highest number is returned.
+#'
 #' * `hasUnbrokenLoops(x)` returns TRUE if `x` has loops, otherwise FALSE. (No
 #' computation is done here; the function simply returns the value of
 #' `x$UNBROKEN_LOOPS`).
@@ -48,6 +51,7 @@
 #' x = fullSibMating(1)
 #' stopifnot(pedsize(x) == 6)
 #' stopifnot(hasUnbrokenLoops(x))
+#' stopifnot(generations(x) == 3)
 #'
 #' # All members have common ancestors except the grandparents
 #' CA = hasCommonAncestor(x)
@@ -81,6 +85,30 @@ pedsize = function(x) {
   stop2("Input to `pedsize()` must be a `ped` object or a list of such")
 }
 
+#' @rdname ped_utils
+#' @export
+generations = function(x, maxComp = TRUE) {
+
+  if(is.pedList(x)) {
+    gens = sapply(x, generations)
+    return(if(maxComp) max(gens) else gens)
+  }
+
+  x = parentsBeforeChildren(x)
+
+  FIDX = x$FIDX
+  MIDX = x$MIDX
+  NONFOU = nonfounders(x, internal = TRUE)
+  N = pedsize(x)
+
+  # Vector of (maximal) generation number of each ID: dp[i] = 1 + max(dp[parents])
+  # NB: Requires "parentsBeforeChildren".
+  dp = rep(1L, N)
+  for(i in NONFOU)
+    dp[i] = 1L + max(dp[c(FIDX[i], MIDX[i])])
+
+  max(dp)
+}
 
 #' @rdname ped_utils
 #' @export
@@ -363,3 +391,75 @@ getComponent = function(x, ids, checkUnique = FALSE, errorIfUnknown = FALSE) {
   # Return comp idx of the input ids, including NA if not present
   compi[idx]
 }
+
+
+
+topolOrder = function(x) {
+  N = pedsize(x)
+
+  env = new.env()
+  logvec = logical(N)
+  names(logvec) = x$ID
+
+  isNonFou = logvec
+  isNonFou[nonfounders(x)] = TRUE
+
+  env$visited = logvec
+  env$ord = character(N)
+  env$k = 1
+
+  dfs = function(x, id, env) {
+    env$visited[id] = TRUE
+
+    if(isNonFou[id]) {
+      fa = father(x, id)
+      if(!env$visited[fa])
+        dfs(x, fa, env)
+      mo = mother(x, id)
+      if(!env$visited[mo])
+        dfs(x, mo, env)
+    }
+
+    k = env$k
+    env$ord[k] = id
+    #message("Added ", id, " in position ", N-k)
+
+    env$k = k + 1
+  }
+
+  lvs = sort.int(leaves(x), method = "quick")
+  for(id in lvs)
+    dfs(x, id, env)
+
+  # Return ordered labels
+  env$ord
+}
+
+reorderSimple = function(x, neworder, ids = NULL) {
+  if(!is.ped(x))
+    stop2("`reorderQuick` only works for `ped` objects")
+  if(length(neworder) != length(x$ID))
+    stop2("Wrong length of `neworder`")
+
+  if(is.character(neworder))
+    neworder = match(neworder, x$ID)
+
+  ID = x$ID[neworder]
+  FIDX = x$FIDX[neworder]
+  MIDX = x$MIDX[neworder]
+  SEX = x$SEX[neworder]
+
+  nonf = FIDX > 0
+  FIDX[nonf] = match(FIDX[nonf], neworder)
+  MIDX[nonf] = match(MIDX[nonf], neworder)
+
+  # Mask all but `ids` indivs
+  if(!is.null(ids)) {
+    mask = !ID %in% ids
+    ID[mask] = "*"
+    SEX[mask] = "*"
+  }
+
+  list(ID, FIDX, MIDX, SEX)
+}
+
